@@ -62,7 +62,7 @@ static const char *completion_find(const char *key, int automatic)
 }
 
 /* Return whole word at specified position in string */
-char *get_word_at(const char *str, int pos, char **startpos)
+static char *get_word_at(const char *str, int pos, char **startpos)
 {
 	const char *start, *end;
 
@@ -128,7 +128,7 @@ static void free_completions(void)
 }
 
 /* manual word completion - called when TAB is pressed */
-char *word_complete(WINDOW_REC *window, const char *line, int *pos, int erase)
+char *word_complete(WINDOW_REC *window, const char *line, int *pos, int erase, int backward)
 {
 	static int startpos = 0, wordlen = 0;
         int old_startpos, old_wordlen;
@@ -143,6 +143,9 @@ char *word_complete(WINDOW_REC *window, const char *line, int *pos, int erase)
 	continue_complete = complist != NULL && *pos == last_line_pos &&
 		strcmp(line, last_line) == 0;
 
+	if (erase && !continue_complete)
+		return NULL;
+
 	old_startpos = startpos;
 	old_wordlen = wordlen;
 
@@ -193,20 +196,19 @@ char *word_complete(WINDOW_REC *window, const char *line, int *pos, int erase)
 	if (erase) {
 		signal_emit("complete erase", 3, window, word, linestart);
 
-		if (!continue_complete)
-                        return NULL;
-
                 /* jump to next completion */
-		word = NULL;
-		linestart = NULL;
                 startpos = old_startpos;
 		wordlen = old_wordlen;
 	}
 
 	if (continue_complete) {
 		/* complete from old list */
-		complist = complist->next != NULL ? complist->next :
-			g_list_first(complist);
+		if (backward)
+			complist = complist->prev != NULL ? complist->prev :
+				g_list_last(complist);
+		else
+			complist = complist->next != NULL ? complist->next :
+				g_list_first(complist);
 		want_space = last_want_space;
 	} else {
 		/* get new completion list */
@@ -255,7 +257,7 @@ char *word_complete(WINDOW_REC *window, const char *line, int *pos, int erase)
 	((!g_path_is_absolute(path) || IS_CURRENT_DIR(path)) && \
 	 default_path != NULL)
 
-GList *list_add_file(GList *list, const char *name, const char *default_path)
+static GList *list_add_file(GList *list, const char *name, const char *default_path)
 {
 	struct stat statbuf;
 	char *fname;
@@ -299,7 +301,7 @@ GList *filename_complete(const char *path, const char *default_path)
 	}
 
 	/* open directory for reading */
-	dir = g_dirname(realpath);
+	dir = g_path_get_dirname(realpath);
 	dirp = opendir(dir);
 	g_free(dir);
         g_free(realpath);
@@ -307,7 +309,7 @@ GList *filename_complete(const char *path, const char *default_path)
 	if (dirp == NULL)
 		return NULL;
 
-	dir = g_dirname(path);
+	dir = g_path_get_dirname(path);
 	if (*dir == G_DIR_SEPARATOR && dir[1] == '\0') {
                 /* completing file in root directory */
 		*dir = '\0';
@@ -463,7 +465,7 @@ static GList *completion_get_subcommands(const char *cmd)
 	return complist;
 }
 
-GList *completion_get_options(const char *cmd, const char *option)
+static GList *completion_get_options(const char *cmd, const char *option)
 {
 	COMMAND_REC *rec;
 	GList *list;
