@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -27,17 +27,23 @@
 #include "special-vars.h"
 #include "settings.h"
 
-#include "window-items.h"
-#include "fe-queries.h"
+#include "servers.h"
 #include "channels.h"
 #include "nicklist.h"
-#include "hilight-text.h"
 #include "ignore.h"
+
+#include "window-items.h"
+#include "fe-queries.h"
+#include "hilight-text.h"
 #include "printtext.h"
 
-#define ishighalnum(c) ((unsigned char) (c) >= 128 || isalnum(c))
+#define ishighalnum(c) ((unsigned char) (c) >= 128 || i_isalnum(c))
+#define isnickchar(a) \
+	(i_isalnum(a) || (a) == '`' || (a) == '-' || (a) == '_' || \
+	(a) == '[' || (a) == ']' || (a) == '{' || (a) == '}' || \
+	(a) == '|' || (a) == '\\' || (a) == '^')
 
-static GHashTable *printnicks;
+GHashTable *printnicks;
 
 /* convert _underlined_ and *bold* words (and phrases) to use real
    underlining or bolding */
@@ -65,7 +71,7 @@ char *expand_emphasis(WI_ITEM_REC *item, const char *text)
 
 		/* check that the beginning marker starts a word, and
 		   that the matching end marker ends a word */
-		if ((pos > 0 && !isspace(bgn[-1])) || !ishighalnum(bgn[1]))
+		if ((pos > 0 && bgn[-1] != ' ') || !ishighalnum(bgn[1]))
 			continue;
 		if ((end = strchr(bgn+1, *bgn)) == NULL)
 			continue;
@@ -78,12 +84,25 @@ char *expand_emphasis(WI_ITEM_REC *item, const char *text)
 			   use emphasis on them. */
 			int found;
                         char c;
+			char *end2; 
 
+			/* check if _foo_ is a nick */
 			c = end[1];
                         end[1] = '\0';
                         found = nicklist_find(CHANNEL(item), bgn) != NULL;
 			end[1] = c;
 			if (found) continue;
+			
+			/* check if the whole 'word' (e.g. "_foo_^") is a nick
+			   in "_foo_^ ", end will be the second _, end2 the ^ */
+			end2 = end;
+			while (isnickchar(end2[1]))
+				end2++;
+			c = end2[1];
+			end2[1] = '\0';
+			found = nicklist_find(CHANNEL(item), bgn) != NULL;
+			end2[1] = c;
+			if (found) continue;
 		}
 
 		/* allow only *word* emphasis, not *multiple words* */
@@ -111,35 +130,32 @@ char *expand_emphasis(WI_ITEM_REC *item, const char *text)
 	return ret;
 }
 
-char *channel_get_nickmode(CHANNEL_REC *channel, const char *nick)
-{
-        NICK_REC *nickrec;
-        char *emptystr;
-
-	g_return_val_if_fail(nick != NULL, NULL);
-
-	if (!settings_get_bool("show_nickmode"))
-                return "";
-
-        emptystr = settings_get_bool("show_nickmode_empty") ? " " : "";
-
-	nickrec = channel == NULL ? NULL :
-		nicklist_find(channel, nick);
-	return nickrec == NULL ? emptystr :
-		(nickrec->op ? "@" : (nickrec->voice ? "+" : emptystr));
-}
-
 static char *channel_get_nickmode_rec(NICK_REC *nickrec)
 {
         char *emptystr;
+	char *nickmode;
 
 	if (!settings_get_bool("show_nickmode"))
-                return "";
+                return g_strdup("");
 
         emptystr = settings_get_bool("show_nickmode_empty") ? " " : "";
 
-	return nickrec == NULL ? emptystr :
-		(nickrec->op ? "@" : (nickrec->voice ? "+" : emptystr));
+	if (nickrec == NULL || nickrec->prefixes[0] == '\0')
+		nickmode = g_strdup(emptystr);
+	else {
+		nickmode = g_malloc(2);
+		nickmode[0] = nickrec->prefixes[0];
+		nickmode[1] = '\0';
+	}
+	return nickmode;
+}
+
+char *channel_get_nickmode(CHANNEL_REC *channel, const char *nick)
+{
+	g_return_val_if_fail(nick != NULL, NULL);
+
+        return channel_get_nickmode_rec(channel == NULL ? NULL :
+					nicklist_find(channel, nick));
 }
 
 static void sig_message_public(SERVER_REC *server, const char *msg,
@@ -147,9 +163,10 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 			       const char *target, NICK_REC *nickrec)
 {
 	CHANNEL_REC *chanrec;
-	const char *nickmode, *printnick;
+	const char *printnick;
 	int for_me, print_channel, level;
-	char *color, *freemsg = NULL;
+	char *nickmode, *color, *freemsg = NULL;
+	HILIGHT_REC *hilight;
 
 	/* NOTE: this may return NULL if some channel is just closed with
 	   /WINDOW CLOSE and server still sends the few last messages */
@@ -157,9 +174,11 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 	if (nickrec == NULL && chanrec != NULL)
                 nickrec = nicklist_find(chanrec, nick);
 
-	for_me = nick_match_msg(chanrec, msg, server->nick);
-	color = for_me ? NULL :
+	for_me = !settings_get_bool("hilight_nick_matches") ? FALSE :
+		nick_match_msg(chanrec, msg, server->nick);
+	hilight = for_me ? NULL :
 		hilight_match_nick(server, target, nick, address, MSGLEVEL_PUBLIC, msg);
+	color = (hilight == NULL) ? NULL : hilight_get_color(hilight);
 
 	print_channel = chanrec == NULL ||
 		!window_item_is_active((WI_ITEM_REC *) chanrec);
@@ -167,8 +186,9 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 	    window_item_window((WI_ITEM_REC *) chanrec)->items->next != NULL)
 		print_channel = TRUE;
 
-	level = MSGLEVEL_PUBLIC | (for_me || color != NULL ?
-				   MSGLEVEL_HILIGHT : MSGLEVEL_NOHILIGHT);
+	level = MSGLEVEL_PUBLIC;
+	if (for_me)
+		level |= MSGLEVEL_HILIGHT;
 
 	if (settings_get_bool("emphasis"))
 		msg = freemsg = expand_emphasis((WI_ITEM_REC *) chanrec, msg);
@@ -181,34 +201,32 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 	if (printnick == NULL)
 		printnick = nick;
 
-	if (!print_channel) {
-		/* message to active channel in window */
-		if (color != NULL) {
-			/* highlighted nick */
-			printformat(server, target, level,
-				    TXT_PUBMSG_HILIGHT,
-				    color, printnick, msg, nickmode);
-		} else {
+	if (color != NULL) {
+		/* highlighted nick */
+		TEXT_DEST_REC dest;
+		format_create_dest(&dest, server, target, level, NULL);
+		hilight_update_text_dest(&dest,hilight);
+		if (!print_channel) /* message to active channel in window */
+			printformat_dest(&dest, TXT_PUBMSG_HILIGHT, color,
+				         printnick, msg, nickmode);
+		else /* message to not existing/active channel */
+			printformat_dest(&dest, TXT_PUBMSG_HILIGHT_CHANNEL,
+					 color, printnick, target, msg,
+					 nickmode);
+	} else {
+		if (!print_channel)
 			printformat(server, target, level,
 				    for_me ? TXT_PUBMSG_ME : TXT_PUBMSG,
 				    printnick, msg, nickmode);
-		}
-	} else {
-		/* message to not existing/active channel */
-		if (color != NULL) {
-			/* highlighted nick */
-			printformat(server, target, level,
-				    TXT_PUBMSG_HILIGHT_CHANNEL,
-				    color, printnick, target, msg, nickmode);
-		} else {
+		else
 			printformat(server, target, level,
 				    for_me ? TXT_PUBMSG_ME_CHANNEL :
 				    TXT_PUBMSG_CHANNEL,
 				    printnick, target, msg, nickmode);
-		}
-	}
+	}						
 
-        g_free_not_null(freemsg);
+	g_free_not_null(nickmode);
+	g_free_not_null(freemsg);
 	g_free_not_null(color);
 }
 
@@ -230,13 +248,17 @@ static void sig_message_private(SERVER_REC *server, const char *msg,
 	g_free_not_null(freemsg);
 }
 
-static void print_own_channel_message(SERVER_REC *server, CHANNEL_REC *channel,
-				      const char *target, const char *msg)
+static void sig_message_own_public(SERVER_REC *server, const char *msg,
+				   const char *target)
 {
 	WINDOW_REC *window;
-	const char *nickmode;
+	CHANNEL_REC *channel;
+	char *nickmode;
         char *freemsg = NULL;
 	int print_channel;
+	channel = channel_find(server, target);
+	if (channel != NULL)
+		target = channel->visible_name;
 
 	nickmode = channel_get_nickmode(channel, server->nick);
 
@@ -261,21 +283,10 @@ static void print_own_channel_message(SERVER_REC *server, CHANNEL_REC *channel,
 			    TXT_OWN_MSG_CHANNEL, server->nick, target, msg, nickmode);
 	}
 
+	g_free_not_null(nickmode);
 	g_free_not_null(freemsg);
 }
 
-static void sig_message_own_public(SERVER_REC *server, const char *msg,
-				   const char *target)
-{
-	CHANNEL_REC *channel;
-
-	g_return_if_fail(server != NULL);
-	g_return_if_fail(msg != NULL);
-
-	channel = channel_find(server, target);
-	print_own_channel_message(server, channel, target, msg);
-}
-
 static void sig_message_own_private(SERVER_REC *server, const char *msg,
 				    const char *target, const char *origtarget)
 {
@@ -284,7 +295,6 @@ static void sig_message_own_private(SERVER_REC *server, const char *msg,
 
 	g_return_if_fail(server != NULL);
 	g_return_if_fail(msg != NULL);
-
 	if (target == NULL) {
 		/* this should only happen if some special target failed and
 		   we should display some error message. currently the special
@@ -350,7 +360,7 @@ static void sig_message_quit(SERVER_REC *server, const char *nick,
 		if (!nicklist_find(rec, nick))
 			continue;
 
-		if (ignore_check(server, nick, address, rec->name,
+		if (ignore_check(server, nick, address, rec->visible_name,
 				 reason, MSGLEVEL_QUITS)) {
 			count++;
 			continue;
@@ -358,17 +368,18 @@ static void sig_message_quit(SERVER_REC *server, const char *nick,
 
 		if (print_channel == NULL ||
 		    active_win->active == (WI_ITEM_REC *) rec)
-			print_channel = rec->name;
+			print_channel = rec->visible_name;
 
 		if (once)
-			g_string_sprintfa(chans, "%s,", rec->name);
+			g_string_append_printf(chans, "%s,", rec->visible_name);
 		else {
 			window = window_item_window((WI_ITEM_REC *) rec);
 			if (g_slist_find(windows, window) == NULL) {
 				windows = g_slist_append(windows, window);
-				printformat(server, rec->name, MSGLEVEL_QUITS,
+				printformat(server, rec->visible_name,
+					    MSGLEVEL_QUITS,
 					    TXT_QUIT, nick, address, reason,
-					    rec->name);
+					    rec->visible_name);
 			}
 		}
 		count++;
@@ -400,7 +411,7 @@ static void sig_message_kick(SERVER_REC *server, const char *channel,
 			     const char *address, const char *reason)
 {
 	printformat(server, channel, MSGLEVEL_KICKS,
-		    TXT_KICK, nick, channel, kicker, reason);
+		    TXT_KICK, nick, channel, kicker, reason, address);
 }
 
 static void print_nick_change_channel(SERVER_REC *server, const char *channel,
@@ -408,13 +419,18 @@ static void print_nick_change_channel(SERVER_REC *server, const char *channel,
 				      const char *address,
 				      int ownnick)
 {
+	int level;
+
 	if (ignore_check(server, oldnick, address,
 			 channel, newnick, MSGLEVEL_NICKS))
 		return;
 
-	printformat(server, channel, MSGLEVEL_NICKS,
+	level = MSGLEVEL_NICKS;
+        if (ownnick) level |= MSGLEVEL_NO_ACT;
+
+	printformat(server, channel, level,
 		    ownnick ? TXT_YOUR_NICK_CHANGED : TXT_NICK_CHANGED,
-		    oldnick, newnick, channel);
+		    oldnick, newnick, channel, address);
 }
 
 static void print_nick_change(SERVER_REC *server, const char *newnick,
@@ -426,7 +442,7 @@ static void print_nick_change(SERVER_REC *server, const char *newnick,
 
 	msgprint = FALSE;
 
-	/* Print to each channel/query where the nick is.
+	/* Print to each channel where the nick is.
 	   Don't print more than once to the same window. */
 	windows = NULL;
 	for (tmp = server->channels; tmp != NULL; tmp = tmp->next) {
@@ -439,30 +455,17 @@ static void print_nick_change(SERVER_REC *server, const char *newnick,
 			continue;
 
 		windows = g_slist_append(windows, window);
-		print_nick_change_channel(server, channel->name, newnick,
-					  oldnick, address, ownnick);
+		print_nick_change_channel(server, channel->visible_name,
+					  newnick, oldnick, address, ownnick);
 		msgprint = TRUE;
 	}
 
-	for (tmp = server->queries; tmp != NULL; tmp = tmp->next) {
-		QUERY_REC *query = tmp->data;
-		WINDOW_REC *window =
-			window_item_window((WI_ITEM_REC *) query);
-
-		if (g_strcasecmp(query->name, oldnick) != 0 ||
-		    g_slist_find(windows, window) != NULL)
-			continue;
-
-		windows = g_slist_append(windows, window);
-		print_nick_change_channel(server, query->name, newnick,
-					  oldnick, address, ownnick);
-		msgprint = TRUE;
-	}
 	g_slist_free(windows);
 
 	if (!msgprint && ownnick) {
 		printformat(server, NULL, MSGLEVEL_NICKS,
-			    TXT_YOUR_NICK_CHANGED, oldnick, newnick, "");
+			    TXT_YOUR_NICK_CHANGED, oldnick, newnick, "",
+			    address);
 	}
 }
 
@@ -475,7 +478,13 @@ static void sig_message_nick(SERVER_REC *server, const char *newnick,
 static void sig_message_own_nick(SERVER_REC *server, const char *newnick,
 				 const char *oldnick, const char *address)
 {
-	print_nick_change(server, newnick, oldnick, address, TRUE);
+        if (!settings_get_bool("show_own_nickchange_once"))
+		print_nick_change(server, newnick, oldnick, address, TRUE);
+	else {
+		printformat(server, NULL, MSGLEVEL_NICKS,
+			    TXT_YOUR_NICK_CHANGED, oldnick, newnick, "",
+			    address);
+	}
 }
 
 static void sig_message_invite(SERVER_REC *server, const char *channel,
@@ -485,7 +494,7 @@ static void sig_message_invite(SERVER_REC *server, const char *channel,
 
 	str = show_lowascii(channel);
 	printformat(server, NULL, MSGLEVEL_INVITES,
-		    TXT_INVITE, nick, str);
+		    TXT_INVITE, nick, str, address);
 	g_free(str);
 }
 
@@ -495,7 +504,7 @@ static void sig_message_topic(SERVER_REC *server, const char *channel,
 {
 	printformat(server, channel, MSGLEVEL_TOPICS,
 		    *topic != '\0' ? TXT_NEW_TOPIC : TXT_TOPIC_UNSET,
-		    nick, channel, topic);
+		    nick, channel, topic, address);
 }
 
 static int printnick_exists(NICK_REC *first, NICK_REC *ignore,
@@ -567,7 +576,7 @@ static void sig_nicklist_new(CHANNEL_REC *channel, NICK_REC *nick)
 	newnick = g_string_new(NULL);
         n = 2;
 	do {
-		g_string_sprintf(newnick, "%s%d", nickhost, n);
+		g_string_printf(newnick, "%s%d", nickhost, n);
                 n++;
 	} while (printnick_exists(firstnick, nick, newnick->str));
 
@@ -630,6 +639,7 @@ void fe_messages_init(void)
 	printnicks = g_hash_table_new((GHashFunc) g_direct_hash,
 				      (GCompareFunc) g_direct_equal);
 
+	settings_add_bool("lookandfeel", "hilight_nick_matches", TRUE);
 	settings_add_bool("lookandfeel", "emphasis", TRUE);
 	settings_add_bool("lookandfeel", "emphasis_replace", FALSE);
 	settings_add_bool("lookandfeel", "emphasis_multiword", FALSE);
@@ -637,19 +647,20 @@ void fe_messages_init(void)
 	settings_add_bool("lookandfeel", "show_nickmode_empty", TRUE);
 	settings_add_bool("lookandfeel", "print_active_channel", FALSE);
 	settings_add_bool("lookandfeel", "show_quit_once", FALSE);
+	settings_add_bool("lookandfeel", "show_own_nickchange_once", FALSE);
 
-	signal_add("message public", (SIGNAL_FUNC) sig_message_public);
-	signal_add("message private", (SIGNAL_FUNC) sig_message_private);
-	signal_add("message own_public", (SIGNAL_FUNC) sig_message_own_public);
-	signal_add("message own_private", (SIGNAL_FUNC) sig_message_own_private);
-	signal_add("message join", (SIGNAL_FUNC) sig_message_join);
-	signal_add("message part", (SIGNAL_FUNC) sig_message_part);
-	signal_add("message quit", (SIGNAL_FUNC) sig_message_quit);
-	signal_add("message kick", (SIGNAL_FUNC) sig_message_kick);
-	signal_add("message nick", (SIGNAL_FUNC) sig_message_nick);
-	signal_add("message own_nick", (SIGNAL_FUNC) sig_message_own_nick);
-	signal_add("message invite", (SIGNAL_FUNC) sig_message_invite);
-	signal_add("message topic", (SIGNAL_FUNC) sig_message_topic);
+	signal_add_last("message public", (SIGNAL_FUNC) sig_message_public);
+	signal_add_last("message private", (SIGNAL_FUNC) sig_message_private);
+	signal_add_last("message own_public", (SIGNAL_FUNC) sig_message_own_public);
+	signal_add_last("message own_private", (SIGNAL_FUNC) sig_message_own_private);
+	signal_add_last("message join", (SIGNAL_FUNC) sig_message_join);
+	signal_add_last("message part", (SIGNAL_FUNC) sig_message_part);
+	signal_add_last("message quit", (SIGNAL_FUNC) sig_message_quit);
+	signal_add_last("message kick", (SIGNAL_FUNC) sig_message_kick);
+	signal_add_last("message nick", (SIGNAL_FUNC) sig_message_nick);
+	signal_add_last("message own_nick", (SIGNAL_FUNC) sig_message_own_nick);
+	signal_add_last("message invite", (SIGNAL_FUNC) sig_message_invite);
+	signal_add_last("message topic", (SIGNAL_FUNC) sig_message_topic);
 
 	signal_add("nicklist new", (SIGNAL_FUNC) sig_nicklist_new);
 	signal_add("nicklist remove", (SIGNAL_FUNC) sig_nicklist_remove);
