@@ -31,7 +31,6 @@
 #include "window-items.h"
 #include "formats.h"
 #include "themes.h"
-#include "translation.h"
 #include "recode.h"
 #include "utf8.h"
 
@@ -82,22 +81,6 @@ static void format_expand_code(const char **format, GString *out, int *flags)
 		else if (**format == '-')
 			set = FALSE;
 		else switch (**format) {
-		case 'i':
-			/* indent function */
-			(*format)++;
-			if (**format == '=')
-				(*format)++;
-
-			g_string_append_c(out, 4);
-			g_string_append_c(out, FORMAT_STYLE_INDENT_FUNC);
-			while (**format != ']' && **format != '\0' &&
-			       **format != ',') {
-				g_string_append_c(out, **format);
-				(*format)++;
-			}
-			g_string_append_c(out, ',');
-			(*format)--;
-			break;
 		case 's':
 		case 'S':
 			*flags |= !set ? PRINT_FLAG_UNSET_LINE_START :
@@ -829,10 +812,10 @@ static void get_mirc_color(const char **str, int *fg_ret, int *bg_ret)
 		}
 		if (**str == ',') {
 			/* background color */
-			(*str)++;
-			if (!i_isdigit(**str))
+			if (!i_isdigit((*str)[1]))
 				bg = -1;
 			else {
+				(*str)++;
 				bg = **str-'0';
 				(*str)++;
 				if (i_isdigit(**str)) {
@@ -949,8 +932,7 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 	int fgcolor, bgcolor;
 	int flags;
 
-	theme = dest->window != NULL && dest->window->theme != NULL ?
-		dest->window->theme : current_theme;
+	theme = window_get_theme(dest->window);
 
 	dup = str = g_strdup(text);
 
@@ -963,8 +945,6 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 				*ptr++ = '\0';
 				break;
 			}
-
-			*ptr = (char) translation_in[(int) (unsigned char) *ptr];
 		}
 
 		if (type == 7) {
@@ -986,8 +966,12 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 			flags &= ~(GUI_PRINT_FLAG_INDENT|GUI_PRINT_FLAG_CLRTOEOL);
 		}
 
-		if (type == '\n')
+		if (type == '\n') {
 			format_newline(dest->window);
+			fgcolor = theme->default_color;
+			bgcolor = -1;
+			flags &= GUI_PRINT_FLAG_INDENT|GUI_PRINT_FLAG_MONOSPACE;
+		}
 
 		if (*ptr == '\0')
 			break;
@@ -1029,18 +1013,6 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 			case FORMAT_STYLE_INDENT:
 				flags |= GUI_PRINT_FLAG_INDENT;
 				break;
-			case FORMAT_STYLE_INDENT_FUNC: {
-				const char *start = ptr;
-				while (*ptr != ',' && *ptr != '\0')
-					ptr++;
-				if (*ptr != '\0') *ptr++ = '\0';
-				ptr--;
-				signal_emit_id(signal_gui_print_text, 6,
-					       dest->window, NULL, NULL,
-					       GINT_TO_POINTER(GUI_PRINT_FLAG_INDENT_FUNC),
-					       start, dest);
-				break;
-			}
 			case FORMAT_STYLE_DEFAULTS:
 				fgcolor = theme->default_color;
 				bgcolor = -1;
@@ -1051,13 +1023,6 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 			default:
 				if (*ptr != FORMAT_COLOR_NOCHANGE) {
 					fgcolor = (unsigned char) *ptr-'0';
-					if (fgcolor <= 7)
-						flags &= ~GUI_PRINT_FLAG_BOLD;
-					else {
-						/* bold */
-						if (fgcolor != 8) fgcolor -= 8;
-						flags |= GUI_PRINT_FLAG_BOLD;
-					}
 				}
 				if (ptr[1] == '\0')
 					break;
@@ -1065,13 +1030,6 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 				ptr++;
 				if (*ptr != FORMAT_COLOR_NOCHANGE) {
 					bgcolor = *ptr-'0';
-					if (bgcolor <= 7)
-						flags &= ~GUI_PRINT_FLAG_BLINK;
-					else {
-						/* blink */
-						bgcolor -= 8;
-						flags |= GUI_PRINT_FLAG_BLINK;
-					}
 				}
 			}
 			ptr++;
