@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -140,10 +140,14 @@ static void event_part(IRC_SERVER_REC *server, const char *data,
 static void event_quit(IRC_SERVER_REC *server, const char *data,
 		       const char *nick, const char *addr)
 {
+	char *recoded;
+
 	g_return_if_fail(data != NULL);
 
 	if (*data == ':') data++; /* quit message */
-	signal_emit("message quit", 4, server, nick, addr, data);
+	recoded = recode_in(SERVER(server), data, nick);
+	signal_emit("message quit", 4, server, nick, addr, recoded);
+	g_free(recoded);
 }
 
 static void event_kick(IRC_SERVER_REC *server, const char *data,
@@ -285,7 +289,7 @@ static void event_wallops(IRC_SERVER_REC *server, const char *data, const char *
 	if (ignore_check(SERVER(server), nick, addr, NULL, data, MSGLEVEL_WALLOPS))
 		return;
 
-	if (g_strncasecmp(data, "\001ACTION ", 8) != 0)
+	if (g_ascii_strncasecmp(data, "\001ACTION ", 8) != 0)
 		printformat(server, NULL, MSGLEVEL_WALLOPS, IRCTXT_WALLOPS, nick, data);
 	else {
 		/* Action in WALLOP */
@@ -294,7 +298,7 @@ static void event_wallops(IRC_SERVER_REC *server, const char *data, const char *
 
 		tmp = g_strdup(data+8);
 		len = strlen(tmp);
-		if (tmp[len-1] == 1) tmp[len-1] = '\0';
+		if (len >= 1 && tmp[len-1] == 1) tmp[len-1] = '\0';
 		printformat(server, NULL, MSGLEVEL_WALLOPS, IRCTXT_ACTION_WALLOPS, nick, tmp);
 		g_free(tmp);
 	}
