@@ -6,6 +6,7 @@
 #include "levels.h"
 #include "misc.h"
 #include "settings.h"
+#include "recode.h"
 
 #include "irc-servers.h"
 
@@ -13,15 +14,17 @@
 
 static void event_whois(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *nick, *user, *host, *realname;
+	char *params, *nick, *user, *host, *realname, *recoded;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 6, NULL, &nick, &user,
 				  &host, NULL, &realname);
+	recoded = recode_in(SERVER(server), realname, nick);
 	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS, nick, user, host, realname);
+		    IRCTXT_WHOIS, nick, user, host, recoded);
 	g_free(params);
+	g_free(recoded);
 }
 
 static void event_whois_special(IRC_SERVER_REC *server, const char *data)
@@ -113,8 +116,10 @@ static void event_whois_modes(IRC_SERVER_REC *server, const char *data)
 
 	g_return_if_fail(data != NULL);
 
-	params = event_get_params(data, 6, NULL, &nick,
-				  NULL, NULL, NULL, &modes);
+	params = event_get_params(data, 3 | PARAM_FLAG_GETREST,
+			NULL, &nick, &modes);
+	if (!strncmp(modes, "is using modes ", 15))
+		modes += 15;
 	printformat(server, nick, MSGLEVEL_CRAP,
 		    IRCTXT_WHOIS_MODES, nick, modes);
 	g_free(params);
@@ -139,7 +144,9 @@ static void event_whois_realhost(IRC_SERVER_REC *server, const char *data)
                 if (hostname != NULL) hostname += 5;
 	}
 
-	if (hostname == NULL) {
+	if (hostname != NULL) {
+		if (!strncmp(hostname, "*@", 2))
+			hostname += 2;
 		printformat(server, nick, MSGLEVEL_CRAP,
 			    IRCTXT_WHOIS_REALHOST, nick, hostname, "");
 	} else {
@@ -178,6 +185,31 @@ static void event_whois_realhost327(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
+static void event_whois_realhost338(IRC_SERVER_REC *server, const char *data)
+{
+	char *params, *nick, *arg1, *arg2, *arg3;
+
+	g_return_if_fail(data != NULL);
+
+	/*
+	 * :<server> 338 <yournick> <nick> <user>@<host> <ip> :Actual user@host, actual IP
+	 * (ircu) or
+	 * :<server> 338 <yournick> <nick> <ip> :actually using host
+	 * (ratbox)
+	 */
+	params = event_get_params(data, 5, NULL, &nick, &arg1, &arg2, &arg3);
+	if (*arg3 != '\0') {
+		printformat(server, nick, MSGLEVEL_CRAP,
+			    IRCTXT_WHOIS_REALHOST, nick, arg1, arg2);
+	} else if (*arg2 != '\0') {
+		printformat(server, nick, MSGLEVEL_CRAP,
+			    IRCTXT_WHOIS_REALHOST, nick, arg1, "");
+	} else {
+		event_whois_special(server, data);
+	}
+	g_free(params);
+}
+
 static void event_whois_usermode(IRC_SERVER_REC *server, const char *data)
 {
 	char *params, *txt_usermodes, *nick, *usermode;
@@ -246,7 +278,7 @@ static void hide_safe_channel_id(IRC_SERVER_REC *server, char *chans)
 
 static void event_whois_channels(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *nick, *chans;
+	char *params, *nick, *chans, *recoded;
 
 	g_return_if_fail(data != NULL);
 
@@ -259,23 +291,27 @@ static void event_whois_channels(IRC_SERVER_REC *server, const char *data)
 	chans = show_lowascii(chans);
 	if (settings_get_bool("whois_hide_safe_channel_id"))
 		hide_safe_channel_id(server, chans);
+	recoded = recode_in(SERVER(server), chans, nick);
 	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_CHANNELS, nick, chans);
+		    IRCTXT_WHOIS_CHANNELS, nick, recoded);
 	g_free(chans);
 
 	g_free(params);
+	g_free(recoded);
 }
 
 static void event_whois_away(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *nick, *awaymsg;
+	char *params, *nick, *awaymsg, *recoded;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 3, NULL, &nick, &awaymsg);
+	recoded = recode_in(SERVER(server), awaymsg, nick);
 	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_AWAY, nick, awaymsg);
+		    IRCTXT_WHOIS_AWAY, nick, recoded);
 	g_free(params);
+	g_free(recoded);
 }
 
 static void event_end_of_whois(IRC_SERVER_REC *server, const char *data)
@@ -306,15 +342,17 @@ static void event_whois_auth(IRC_SERVER_REC *server, const char *data)
 
 static void event_whowas(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *nick, *user, *host, *realname;
+	char *params, *nick, *user, *host, *realname, *recoded;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 6, NULL, &nick, &user,
 				  &host, NULL, &realname);
+	recoded = recode_in(SERVER(server), realname, nick);
 	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOWAS, nick, user, host, realname);
+		    IRCTXT_WHOWAS, nick, user, host, recoded);
 	g_free(params);
+	g_free(recoded);
 }
 
 static void event_end_of_whowas(IRC_SERVER_REC *server, const char *data)
@@ -340,6 +378,7 @@ static struct whois_event_table events[] = {
 	{ 312, event_whois_server },
 	{ 326, event_whois_usermode326 },
 	{ 327, event_whois_realhost327 },
+	{ 338, event_whois_realhost338 },
 	{ 379, event_whois_modes },
 	{ 378, event_whois_realhost },
 	{ 377, event_whois_usermode },
@@ -381,6 +420,7 @@ void fe_whois_init(void)
 	signal_add("event 379", (SIGNAL_FUNC) event_whois_modes);
 	signal_add("event 327", (SIGNAL_FUNC) event_whois_realhost327);
 	signal_add("event 326", (SIGNAL_FUNC) event_whois_usermode326);
+	signal_add("event 338", (SIGNAL_FUNC) event_whois_realhost338);
 	signal_add("whois away", (SIGNAL_FUNC) event_whois_away);
 	signal_add("whois oper", (SIGNAL_FUNC) event_whois_oper);
 	signal_add("whowas away", (SIGNAL_FUNC) event_whois_away);
@@ -403,6 +443,7 @@ void fe_whois_deinit(void)
 	signal_remove("event 379", (SIGNAL_FUNC) event_whois_modes);
 	signal_remove("event 327", (SIGNAL_FUNC) event_whois_realhost327);
 	signal_remove("event 326", (SIGNAL_FUNC) event_whois_usermode326);
+	signal_remove("event 338", (SIGNAL_FUNC) event_whois_realhost338);
 	signal_remove("whois away", (SIGNAL_FUNC) event_whois_away);
 	signal_remove("whois oper", (SIGNAL_FUNC) event_whois_oper);
 	signal_remove("whowas away", (SIGNAL_FUNC) event_whois_away);
