@@ -58,8 +58,7 @@ void ctcp_register(const char *name)
 	rec = ctcp_cmd_find(name);
 	if (rec == NULL) {
 		rec = g_new0(CTCP_CMD_REC, 1);
-		rec->name = g_strdup(name);
-		g_strup(rec->name);
+		rec->name = g_ascii_strup(name, -1);
 
 		ctcp_cmds = g_slist_append(ctcp_cmds, rec);
 	}
@@ -207,7 +206,7 @@ static void ctcp_clientinfo(IRC_SERVER_REC *server, const char *data,
 	g_return_if_fail(nick != NULL);
 
 	str = g_string_new(NULL);
-        g_string_sprintf(str, "NOTICE %s :\001CLIENTINFO", nick);
+        g_string_printf(str, "NOTICE %s :\001CLIENTINFO", nick);
 	for (tmp = ctcp_cmds; tmp != NULL; tmp = tmp->next) {
 		CTCP_CMD_REC *rec = tmp->data;
 
@@ -225,7 +224,7 @@ static void ctcp_msg(IRC_SERVER_REC *server, const char *data,
 {
 	char *args, *str;
 
-	if (g_strncasecmp(data, "ACTION ", 7) == 0) {
+	if (g_ascii_strncasecmp(data, "ACTION ", 7) == 0) {
                 /* special treatment for actions */
 		signal_emit("ctcp action", 5, server, data+7,
 			    nick, addr, target);
