@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -44,8 +44,8 @@ void irc_channels_setup_deinit(void);
 void massjoin_init(void);
 void massjoin_deinit(void);
 
-IRC_CHANNEL_REC *irc_channel_create(IRC_SERVER_REC *server,
-				    const char *name, int automatic)
+IRC_CHANNEL_REC *irc_channel_create(IRC_SERVER_REC *server, const char *name,
+				    const char *visible_name, int automatic)
 {
 	IRC_CHANNEL_REC *rec;
 
@@ -53,12 +53,10 @@ IRC_CHANNEL_REC *irc_channel_create(IRC_SERVER_REC *server,
 	g_return_val_if_fail(name != NULL, NULL);
 
 	rec = g_new0(IRC_CHANNEL_REC, 1);
-	rec->chat_type = IRC_PROTOCOL;
-	rec->name = g_strdup(name);
-	rec->server = server;
 	if (*name == '+') rec->no_modes = TRUE;
 
-	channel_init((CHANNEL_REC *) rec, automatic);
+	channel_init((CHANNEL_REC *) rec, (SERVER_REC *) server,
+		     name, visible_name, automatic);
 	return rec;
 }
 
@@ -71,10 +69,10 @@ static void irc_channels_join(IRC_SERVER_REC *server, const char *data,
 	CHANNEL_SETUP_REC *schannel;
 	IRC_CHANNEL_REC *chanrec;
 	GString *outchans, *outkeys;
-	char *channels, *keys, *key;
-	char **chanlist, **keylist, **tmp, **tmpkey, *channel, *channame;
+	char *channels, *keys, *key, *space;
+	char **chanlist, **keylist, **tmp, **tmpkey, **tmpstr, *channel, *channame;
 	void *free_arg;
-	int use_keys;
+	int use_keys, cmdlen;
 
 	g_return_if_fail(data != NULL);
 	g_return_if_fail(IS_IRC_SERVER(server) && server->connected);
@@ -84,6 +82,12 @@ static void irc_channels_join(IRC_SERVER_REC *server, const char *data,
 			    &channels, &keys))
 		return;
 
+	/* keys shouldn't contain space */
+	space = strchr(keys, ' ');
+	if (space != NULL) {
+		*space = '\0';
+	}
+
         chanlist = g_strsplit(channels, ",", -1);
 	keylist = g_strsplit(keys, ",", -1);
 
@@ -92,44 +96,68 @@ static void irc_channels_join(IRC_SERVER_REC *server, const char *data,
 
 	use_keys = *keys != '\0';
 	tmpkey = keylist;
-	for (tmp = chanlist; *tmp != NULL; tmp++) {
-		channel = ischannel(**tmp) ? g_strdup(*tmp) :
+	tmp = chanlist;
+	for (;; tmp++) {
+		if (*tmp !=  NULL) {
+			channel = ischannel(**tmp) ? g_strdup(*tmp) :
 			g_strdup_printf("#%s", *tmp);
 
-		chanrec = irc_channel_find(server, channel);
-		if (chanrec == NULL) {
-			schannel = channel_setup_find(channel, server->connrec->chatnet);
+			chanrec = irc_channel_find(server, channel);
+			if (chanrec == NULL) {
+				schannel = channel_setup_find(channel, server->connrec->chatnet);
 
-                        g_string_sprintfa(outchans, "%s,", channel);
-                        if (*tmpkey != NULL && **tmpkey != '\0')
-                        	key = *tmpkey;
-                        else if (schannel != NULL && schannel->password != NULL) {
-				/* get password from setup record */
-                                use_keys = TRUE;
-				key = schannel->password;
-			} else key = NULL;
+				g_string_append_printf(outchans, "%s,", channel);
+				if (*tmpkey != NULL && **tmpkey != '\0')
+                        		key = *tmpkey;
+	                        else if (schannel != NULL && schannel->password != NULL) {
+					/* get password from setup record */
+                	                use_keys = TRUE;
+					key = schannel->password;
+				} else key = NULL;
+				
+				g_string_append_printf(outkeys, "%s,", get_join_key(key));
+				channame = channel + (channel[0] == '!' &&
+						      channel[1] == '!');
+				chanrec = irc_channel_create(server, channame, NULL,
+							     automatic);
+				if (key != NULL) chanrec->key = g_strdup(key);
+			}
+			g_free(channel);
 
-			g_string_sprintfa(outkeys, "%s,", get_join_key(key));
-			channame = channel + (channel[0] == '!' &&
-					      channel[1] == '!');
-			chanrec = irc_channel_create(server, channame,
-						     automatic);
-			if (key != NULL) chanrec->key = g_strdup(key);
+			if (*tmpkey != NULL)
+                	        tmpkey++;
+	
+			tmpstr = tmp;
+			tmpstr++;
+			cmdlen = outchans->len-1;
+			
+			if (use_keys)
+				cmdlen += outkeys->len;
+			if (*tmpstr != NULL)
+				cmdlen += ischannel(**tmpstr) ? strlen(*tmpstr) :
+					  strlen(*tmpstr)+1;
+			if (*tmpkey != NULL)
+				cmdlen += strlen(*tmpkey);
+				
+			/* don't try to send too long lines 
+			   make sure it's not longer than 510
+			   so 510 - strlen("JOIN ") = 505 */
+			if (cmdlen < 505)
+				continue;
 		}
-		g_free(channel);
-
-		if (*tmpkey != NULL)
-                        tmpkey++;
+		if (outchans->len > 0) {
+			g_string_truncate(outchans, outchans->len-1);
+			g_string_truncate(outkeys, outkeys->len-1);
+			irc_send_cmdv(IRC_SERVER(server),
+				      use_keys ? "JOIN %s %s" : "JOIN %s",
+				      outchans->str, outkeys->str);
+		}
+		cmdlen = 0;
+		g_string_truncate(outchans,0);
+		g_string_truncate(outkeys,0);
+		if (*tmp == NULL || tmp[1] == NULL)
+			break;
 	}
-
-	if (outchans->len > 0) {
-		g_string_truncate(outchans, outchans->len-1);
-		g_string_truncate(outkeys, outkeys->len-1);
-		irc_send_cmdv(IRC_SERVER(server),
-			      use_keys ? "JOIN %s %s" : "JOIN %s",
-			      outchans->str, outkeys->str);
-	}
-
 	g_string_free(outchans, TRUE);
 	g_string_free(outkeys, TRUE);
 
@@ -151,19 +179,18 @@ static CHANNEL_REC *irc_channel_find_server(SERVER_REC *server,
 		if (rec->chat_type != server->chat_type)
                         continue;
 
-		if (g_strcasecmp(channel, rec->name) == 0)
+		/* check both !ABCDEchannel and !channel */
+		if (IRC_SERVER(server)->nick_comp_func(channel, rec->name) == 0)
 			return rec;
 
-                /* check after removing ABCDE from !ABCDEchannel */
-		if (*channel == '!' && *rec->name == '!' &&
-		    g_strcasecmp(channel+1, rec->name+6) == 0)
+		if (IRC_SERVER(server)->nick_comp_func(channel, rec->visible_name) == 0)
 			return rec;
 	}
 
 	return NULL;
 }
 
-static void sig_server_looking(SERVER_REC *server)
+static void sig_server_connected(SERVER_REC *server)
 {
 	if (!IS_IRC_SERVER(server))
 		return;
@@ -192,7 +219,7 @@ static void sig_channel_destroyed(IRC_CHANNEL_REC *channel)
 	if (!IS_IRC_CHANNEL(channel))
                 return;
 
-	if (channel->server != NULL && !channel->left && !channel->kicked) {
+	if (!channel->server->disconnected && !channel->left && !channel->kicked) {
 		/* destroying channel record without actually
 		   having left the channel yet */
 		signal_emit("command part", 3, "", channel->server, channel);
@@ -201,7 +228,7 @@ static void sig_channel_destroyed(IRC_CHANNEL_REC *channel)
 
 void irc_channels_init(void)
 {
-	signal_add("server looking", (SIGNAL_FUNC) sig_server_looking);
+	signal_add_first("server connected", (SIGNAL_FUNC) sig_server_connected);
 	signal_add("channel created", (SIGNAL_FUNC) sig_channel_created);
 	signal_add("channel destroyed", (SIGNAL_FUNC) sig_channel_destroyed);
 
@@ -219,7 +246,7 @@ void irc_channels_init(void)
 
 void irc_channels_deinit(void)
 {
-	signal_remove("server looking", (SIGNAL_FUNC) sig_server_looking);
+	signal_remove("server connected", (SIGNAL_FUNC) sig_server_connected);
 	signal_remove("channel created", (SIGNAL_FUNC) sig_channel_created);
 	signal_remove("channel destroyed", (SIGNAL_FUNC) sig_channel_destroyed);
 
