@@ -78,7 +78,7 @@ static void cmd_notice(const char *data, IRC_SERVER_REC *server,
 		cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
 	recoded = recode_out(SERVER(server), msg, target);
-	g_string_sprintf(tmpstr, "NOTICE %s :%s", target, recoded);
+	g_string_printf(tmpstr, "NOTICE %s :%s", target, recoded);
 	g_free(recoded);
 
 	irc_send_cmd_split(server, tmpstr->str, 2, server->max_msgs_in_cmd);
@@ -106,12 +106,12 @@ static void cmd_ctcp(const char *data, IRC_SERVER_REC *server,
 
 	g_strup(ctcpcmd);
 	if (*ctcpdata == '\0')
-		g_string_sprintf(tmpstr, "PRIVMSG %s :\001%s\001", target, ctcpcmd);
+		g_string_printf(tmpstr, "PRIVMSG %s :\001%s\001", target, ctcpcmd);
 	else {
 		char *recoded;
 
 		recoded = recode_out(SERVER(server), ctcpdata, target);
-		g_string_sprintf(tmpstr, "PRIVMSG %s :\001%s %s\001", target, ctcpcmd, recoded);
+		g_string_printf(tmpstr, "PRIVMSG %s :\001%s %s\001", target, ctcpcmd, recoded);
 		g_free(recoded);
 	}
 
@@ -140,7 +140,7 @@ static void cmd_nctcp(const char *data, IRC_SERVER_REC *server,
 
 	g_strup(ctcpcmd);
 	recoded = recode_out(SERVER(server), ctcpdata, target);
-	g_string_sprintf(tmpstr, "NOTICE %s :\001%s %s\001", target, ctcpcmd, recoded);
+	g_string_printf(tmpstr, "NOTICE %s :\001%s %s\001", target, ctcpcmd, recoded);
 	g_free(recoded);
 
 	irc_send_cmd_split(server, tmpstr->str, 2, server->max_msgs_in_cmd);
@@ -194,7 +194,7 @@ static void cmd_kick(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item
 	if (!ischannel(*channame)) cmd_param_error(CMDERR_NOT_JOINED);
 
 	recoded = recode_out(SERVER(server), reason, channame);
-	g_string_sprintf(tmpstr, "KICK %s %s :%s", channame, nicks, recoded);
+	g_string_printf(tmpstr, "KICK %s %s :%s", channame, nicks, recoded);
 	g_free(recoded);
 
 	irc_send_cmd_split(server, tmpstr->str, 3, server->max_kicks_in_cmd);
@@ -354,14 +354,18 @@ static char *get_redirect_nicklist(const char *nicks, int *free)
 {
 	char *str, *ret;
 
-	if (strchr(nicks, ',') == NULL) {
+	if (*nicks != '!' && strchr(nicks, ',') == NULL) {
 		*free = FALSE;
 		return (char *) nicks;
 	}
 
 	*free = TRUE;
 
-	str = g_strdup(nicks);
+	/* ratbox-style operspy whois takes !nick, echoes that
+	 * in RPL_ENDOFWHOIS as normal but gives output about the
+	 * plain nick
+	 */
+	str = g_strdup(*nicks == '!' ? nicks + 1 : nicks);
 	g_strdelimit(str, ",", ' ');
 	ret = g_strconcat(str, " ", nicks, NULL);
 	g_free(str);
@@ -411,9 +415,9 @@ static void cmd_whois(const char *data, IRC_SERVER_REC *server,
 
 	event_402 = "event 402";
 	if (*qserver == '\0')
-		g_string_sprintf(tmpstr, "WHOIS %s", query);
+		g_string_printf(tmpstr, "WHOIS %s", query);
 	else {
-		g_string_sprintf(tmpstr, "WHOIS %s %s", qserver, query);
+		g_string_printf(tmpstr, "WHOIS %s %s", qserver, query);
 		if (g_strcasecmp(qserver, query) == 0)
 			event_402 = "whois event not found";
 	}
@@ -504,7 +508,7 @@ static void cmd_whowas(const char *data, IRC_SERVER_REC *server)
 	cmd_params_free(free_arg);
 }
 
-/* SYNTAX: PING <nicks> */
+/* SYNTAX: PING [<nick> | <channel> | *] */
 static void cmd_ping(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item)
 {
 	GTimeVal tv;
@@ -512,10 +516,9 @@ static void cmd_ping(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item
 
         CMD_IRC_SERVER(server);
 
-	if (*data == '\0' || strcmp(data, "*") == 0) {
-		if (!IS_IRC_ITEM(item))
-                        cmd_return_error(CMDERR_NOT_JOINED);
-
+	if (*data == '\0') {
+		if (!IS_QUERY(item))
+			cmd_return_error(CMDERR_NOT_ENOUGH_PARAMS);
 		data = window_item_get_target(item);
 	}
 
@@ -526,20 +529,6 @@ static void cmd_ping(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item
 	g_free(str);
 }
 
-static void server_send_away(IRC_SERVER_REC *server, const char *reason)
-{
-	if (!IS_IRC_SERVER(server))
-		return;
-
-	if (*reason != '\0' || server->usermode_away) {
-		g_free_and_null(server->away_reason);
-                if (*reason != '\0')
-			server->away_reason = g_strdup(reason);
-
-		irc_send_cmdv(server, "AWAY :%s", reason);
-	}
-}
-
 /* SYNTAX: AWAY [-one | -all] [<reason>] */
 static void cmd_away(const char *data, IRC_SERVER_REC *server)
 {
@@ -547,15 +536,13 @@ static void cmd_away(const char *data, IRC_SERVER_REC *server)
 	char *reason;
 	void *free_arg;
 
-        CMD_IRC_SERVER(server);
-
 	if (!cmd_get_params(data, &free_arg, 1 | PARAM_FLAG_OPTIONS |
 			    PARAM_FLAG_GETREST, "away", &optlist, &reason)) return;
 
 	if (g_hash_table_lookup(optlist, "one") != NULL)
-		server_send_away(server, reason);
+		irc_server_send_away(server, reason);
 	else
-		g_slist_foreach(servers, (GFunc) server_send_away, reason);
+		g_slist_foreach(servers, (GFunc) irc_server_send_away, reason);
 
 	cmd_params_free(free_arg);
 }
@@ -583,17 +570,6 @@ static void cmd_quote(const char *data, IRC_SERVER_REC *server)
 		irc_send_cmd(server, data);
 }
 
-/* SYNTAX: RAWQUOTE <data> */
-static void cmd_rawquote(const char *data, IRC_SERVER_REC *server)
-{
-	if (server != NULL && !IS_IRC_SERVER(server))
-		return;
-	if (server == NULL || server->connect_time == 0)
-		cmd_return_error(CMDERR_NOT_CONNECTED);
-
-	irc_send_cmd_full(server, data, FALSE, FALSE, TRUE);
-}
-
 static void cmd_wall_hash(gpointer key, NICK_REC *nick, GSList **nicks)
 {
 	if (nick->op) *nicks = g_slist_append(*nicks, nick);
@@ -652,58 +628,35 @@ static void cmd_wall(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item
 	chanrec = irc_channel_find(server, channame);
 	if (chanrec == NULL) cmd_param_error(CMDERR_CHAN_NOT_FOUND);
 
+	recoded = recode_out(SERVER(server), msg, channame);
 	/* See if the server has advertised support of wallchops */
 	if (g_hash_table_lookup(chanrec->server->isupport, "statusmsg") ||
 	    g_hash_table_lookup(chanrec->server->isupport, "wallchops"))
-		irc_send_cmdv(server, "NOTICE @%s :%s", chanrec->name, msg);
+		irc_send_cmdv(server, "NOTICE @%s :%s", chanrec->name, recoded);
 	else {
 		/* Fall back to manually noticing each op */
 		nicks = NULL;
 		g_hash_table_foreach(chanrec->nicks,
 				     (GHFunc) cmd_wall_hash, &nicks);
 
-		args = g_strconcat(chanrec->name, " ", msg, NULL);
+		args = g_strconcat(chanrec->name, " ", recoded, NULL);
 		msg = parse_special_string(settings_get_str("wall_format"),
 					   SERVER(server), item, args, NULL, 0);
 		g_free(args);
 
-		recoded = recode_out(SERVER(server), msg, channame);
-
 		for (tmp = nicks; tmp != NULL; tmp = tmp->next) {
 			NICK_REC *rec = tmp->data;
 
 			if (rec != chanrec->ownnick) {
 				irc_send_cmdv(server, "NOTICE %s :%s",
-					      rec->nick, recoded);
+					      rec->nick, msg);
 			}
 		}
 
-		g_free(recoded);
 		g_free(msg);
 		g_slist_free(nicks);
 	}
 
-	cmd_params_free(free_arg);
-}
-
-/* SYNTAX: WALLCHOPS <channel> <message> */
-/* ircu is the only major server i can see which supports this 
-   and it supports NOTICE @#channel anyway */
-static void cmd_wallchops(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item)
-{
-	char *channame, *msg, *recoded;
-	void *free_arg;
-
-        CMD_IRC_SERVER(server);
-
-	if (!cmd_get_params(data, &free_arg, 2 | PARAM_FLAG_OPTCHAN |
-			    PARAM_FLAG_GETREST, item, &channame, &msg))
-		return;
-	if (*msg == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
-
-	recoded = recode_out(SERVER(server), msg, channame);
-	irc_send_cmdv(server, "WALLCHOPS %s :%s", channame, recoded);
-
 	g_free(recoded);
 	cmd_params_free(free_arg);
 }
@@ -729,8 +682,6 @@ static void cmd_kickban(const char *data, IRC_SERVER_REC *server,
 	chanrec = irc_channel_find(server, channel);
 	if (chanrec == NULL)
 		cmd_param_error(CMDERR_CHAN_NOT_FOUND);
-	if (!chanrec->wholist)
-		cmd_param_error(CMDERR_CHAN_NOT_SYNCED);
 
 	nicklist = g_strsplit(nicks, ",", -1);
         spacenicks = g_strjoinv(" ", nicklist);
@@ -808,8 +759,6 @@ static void cmd_knockout(const char *data, IRC_SERVER_REC *server,
 
 	if (!IS_IRC_CHANNEL(channel))
 		cmd_return_error(CMDERR_NOT_JOINED);
-	if (!channel->wholist)
-		cmd_return_error(CMDERR_CHAN_NOT_SYNCED);
 
 	if (i_isdigit(*data)) {
 		/* first argument is the timeout */
@@ -1076,9 +1025,7 @@ void irc_commands_init(void)
 	/* SYNTAX: USERHOST <nicks> */
 	command_bind_irc("userhost", NULL, (SIGNAL_FUNC) command_self);
 	command_bind_irc("quote", NULL, (SIGNAL_FUNC) cmd_quote);
-	command_bind_irc("rawquote", NULL, (SIGNAL_FUNC) cmd_rawquote);
 	command_bind_irc("wall", NULL, (SIGNAL_FUNC) cmd_wall);
-	command_bind_irc("wallchops", NULL, (SIGNAL_FUNC) cmd_wallchops);
 	command_bind_irc("wait", NULL, (SIGNAL_FUNC) cmd_wait);
 	/* SYNTAX: WALLOPS <message> */
 	command_bind_irc("wallops", NULL, (SIGNAL_FUNC) command_1self);
@@ -1149,9 +1096,7 @@ void irc_commands_deinit(void)
 	command_unbind("uping", (SIGNAL_FUNC) command_self);
 	command_unbind("userhost", (SIGNAL_FUNC) command_self);
 	command_unbind("quote", (SIGNAL_FUNC) cmd_quote);
-	command_unbind("rawquote", (SIGNAL_FUNC) cmd_rawquote);
 	command_unbind("wall", (SIGNAL_FUNC) cmd_wall);
-	command_unbind("wallchops", (SIGNAL_FUNC) cmd_wallchops);
 	command_unbind("wait", (SIGNAL_FUNC) cmd_wait);
 	command_unbind("wallops", (SIGNAL_FUNC) command_1self);
 	command_unbind("kickban", (SIGNAL_FUNC) cmd_kickban);
