@@ -99,7 +99,8 @@ static void event_names_list(IRC_SERVER_REC *server, const char *data)
 	IRC_CHANNEL_REC *chanrec;
 	NICK_REC *rec;
 	char *params, *type, *channel, *names, *ptr;
-        int op, halfop, voice, other;
+        int op, halfop, voice;
+	char prefixes[MAX_USER_PREFIXES+1];
 
 	g_return_if_fail(data != NULL);
 
@@ -137,9 +138,11 @@ static void event_names_list(IRC_SERVER_REC *server, const char *data)
 		/* some servers show ".@nick", there's also been talk about
 		   showing "@+nick" and since none of these chars are valid
 		   nick chars, just check them until a non-nickflag char is
-		   found. FIXME: we just ignore owner char now. */
-		op = halfop = voice = other = FALSE;
+		   found. */
+		op = halfop = voice = FALSE;
+		prefixes[0] = '\0';
 		while (isnickflag(server, *ptr)) {
+			prefix_add(prefixes, *ptr, (SERVER_REC *) server);
 			switch (*ptr) {
 			case '@':
                                 op = TRUE;
@@ -150,8 +153,6 @@ static void event_names_list(IRC_SERVER_REC *server, const char *data)
 			case '+':
                                 voice = TRUE;
                                 break;
-			default:
-				other = *ptr;
 			}
                         ptr++;
 		}
@@ -159,8 +160,7 @@ static void event_names_list(IRC_SERVER_REC *server, const char *data)
 		if (nicklist_find((CHANNEL_REC *) chanrec, ptr) == NULL) {
 			rec = irc_nicklist_insert(chanrec, ptr, op, halfop,
 						  voice, FALSE);
-			if (other)
-				rec->other = other;
+			memcpy(rec->prefixes, prefixes, sizeof(rec->prefixes));
 		}
 	}
 
@@ -409,6 +409,7 @@ static void event_nick(IRC_SERVER_REC *server, const char *data,
 static void event_userhost(SERVER_REC *server, const char *data)
 {
 	char *params, *hosts, **phosts, **pos, *ptr;
+	int oper;
 
 	g_return_if_fail(data != NULL);
 
@@ -418,10 +419,15 @@ static void event_userhost(SERVER_REC *server, const char *data)
 	phosts = g_strsplit(hosts, " ", -1);
 	for (pos = phosts; *pos != NULL; pos++) {
 		ptr = strchr(*pos, '=');
-		if (ptr == NULL) continue;
+		if (ptr == NULL || ptr == *pos) continue;
+		if (ptr[-1] == '*') {
+			ptr[-1] = '\0';
+			oper = 1;
+		} else
+			oper = 0;
 		*ptr++ = '\0';
 
-		nicklist_update_flags(server, *pos, *ptr == '-', -1);
+		nicklist_update_flags(server, *pos, *ptr == '-', oper);
 	}
 	g_strfreev(phosts);
 	g_free(params);
