@@ -70,12 +70,8 @@ static void sig_connected(IRC_SERVER_REC *server)
 	if (!IS_IRC_SERVER(server) || !server->connrec->reconnection)
 		return;
 
-	if (server->connrec->away_reason != NULL) {
-		irc_send_cmdv(server, "AWAY :%s", server->connrec->away_reason);
-
-		g_free(server->away_reason);
-		server->away_reason = g_strdup(server->connrec->away_reason);
-	}
+	if (server->connrec->away_reason != NULL)
+		irc_server_send_away(server, server->connrec->away_reason); 
 }
 
 static void event_nick_collision(IRC_SERVER_REC *server, const char *data)
