@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -49,6 +49,11 @@ static void sig_dcc_request(GET_DCC_REC *dcc, const char *nickaddr)
 	    !masks_match(SERVER(dcc->server), masks, dcc->nick, nickaddr))
 		return;
 
+	/* Unless specifically said in dcc_autoget_masks, don't do autogets
+	   sent to channels. */
+	if (*masks == '\0' && dcc->target != NULL && ischannel(*dcc->target))
+		return;
+
 	/* don't autoget files beginning with a dot, if download dir is
 	   our home dir (stupid kludge for stupid people) */
 	if (*dcc->arg == '.' &&
@@ -57,8 +62,8 @@ static void sig_dcc_request(GET_DCC_REC *dcc, const char *nickaddr)
 
 	/* check file size limit, NOTE: it's still possible to send a
 	   bogus file size and then just send what ever sized file.. */
-        max_size = settings_get_int("dcc_autoget_max_size");
-	if (max_size > 0 && (unsigned long) max_size*1024 < dcc->size)
+        max_size = settings_get_size("dcc_autoget_max_size");
+	if (max_size > 0 && (uoff_t)max_size < dcc->size)
                 return;
 
 	/* ok. but do we want/need to resume? */
@@ -77,7 +82,7 @@ void dcc_autoget_init(void)
 	settings_add_bool("dcc", "dcc_autoget", FALSE);
 	settings_add_bool("dcc", "dcc_autoaccept_lowports", FALSE);
 	settings_add_bool("dcc", "dcc_autoresume", FALSE);
-	settings_add_int("dcc", "dcc_autoget_max_size", 0);
+	settings_add_size("dcc", "dcc_autoget_max_size", "0k");
 	settings_add_str("dcc", "dcc_autoget_masks", "");
 
 	signal_add_last("dcc request", (SIGNAL_FUNC) sig_dcc_request);
