@@ -144,7 +144,7 @@ void proxy_outserver_all_except(CLIENT_REC *client, const char *data, ...)
 static void create_names_start(GString *str, IRC_CHANNEL_REC *channel,
 			       CLIENT_REC *client)
 {
-	g_string_sprintf(str, ":%s 353 %s %c %s :",
+	g_string_printf(str, ":%s 353 %s %c %s :",
 			 client->proxy_address, client->nick,
 			 channel_mode_is_set(channel, 'p') ? '*' :
 			 channel_mode_is_set(channel, 's') ? '@' : '=',
@@ -180,14 +180,8 @@ static void dump_join(IRC_CHANNEL_REC *channel, CLIENT_REC *client)
 		else
 			g_string_append_c(str, ' ');
 
-		if (nick->other)
-                        g_string_append_c(str, nick->other);
-		else if (nick->op)
-                        g_string_append_c(str, '@');
-		else if (nick->halfop)
-                        g_string_append_c(str, '%');
-		else if (nick->voice)
-                        g_string_append_c(str, '+');
+		if (nick->prefixes[0])
+                        g_string_append_c(str, nick->prefixes[0]);
 		g_string_append(str, nick->nick);
 	}
 	g_slist_free(nicks);
@@ -228,9 +222,9 @@ void proxy_client_reset_nick(CLIENT_REC *client)
 static void proxy_dump_data_005(gpointer key, gpointer value, gpointer context)
 {
 	if (*(char *)value != '\0')
-		g_string_sprintfa(context, "%s=%s ", (char *)key, (char *)value);
+		g_string_append_printf(context, "%s=%s ", (char *)key, (char *)value);
 	else
-		g_string_sprintfa(context, "%s ", (char *)key);
+		g_string_append_printf(context, "%s ", (char *)key);
 }
 
 void proxy_dump_data(CLIENT_REC *client)
@@ -242,7 +236,7 @@ void proxy_dump_data(CLIENT_REC *client)
 	proxy_client_reset_nick(client);
 
 	/* welcome info */
-	proxy_outdata(client, ":%s 001 %s :Welcome to the Internet Relay Network\n", client->proxy_address, client->nick);
+	proxy_outdata(client, ":%s 001 %s :Welcome to the Internet Relay Network %s!%s@proxy\n", client->proxy_address, client->nick, client->nick, settings_get_str("user_name"));
 	proxy_outdata(client, ":%s 002 %s :Your host is irssi-proxy, running version %s\n", client->proxy_address, client->nick, PACKAGE_VERSION);
 	proxy_outdata(client, ":%s 003 %s :This server was created ...\n", client->proxy_address, client->nick);
 	if (client->server == NULL || !client->server->emode_known)
@@ -265,7 +259,7 @@ void proxy_dump_data(CLIENT_REC *client)
 
 		for (;; tmp++) {
 			if (*tmp != NULL) {
-				g_string_sprintfa(paramstr, "%s ", *tmp);
+				g_string_append_printf(paramstr, "%s ", *tmp);
 				if (++count < 15)
 					continue;
 			}
@@ -273,10 +267,10 @@ void proxy_dump_data(CLIENT_REC *client)
 			count = 0;
 			if (paramstr->len > 0)
 				g_string_truncate(paramstr, paramstr->len-1);
-			g_string_sprintfa(paramstr, " :are supported by this server\n");
+			g_string_append_printf(paramstr, " :are supported by this server\n");
 			proxy_outdata(client, "%s", paramstr->str);
 			g_string_truncate(paramstr, 0);
-			g_string_sprintf(paramstr, ":%s 005 %s ", client->proxy_address, client->nick);
+			g_string_printf(paramstr, ":%s 005 %s ", client->proxy_address, client->nick);
 
 			if (*tmp == NULL || tmp[1] == NULL)
 				break;
