@@ -50,7 +50,6 @@ static void remove_client(CLIENT_REC *rec)
 	g_free(rec->proxy_address);
 	net_sendbuffer_destroy(rec->handle, TRUE);
 	g_source_remove(rec->recv_tag);
-	line_split_free(rec->buffer);
 	g_free_not_null(rec->nick);
 	g_free_not_null(rec->host);
 	g_free(rec);
@@ -165,7 +164,7 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 	}
 
 	if (strcmp(cmd, "PROXY") == 0) {
-		if (g_strcasecmp(args, "CTCP ON") == 0) {
+		if (g_ascii_strcasecmp(args, "CTCP ON") == 0) {
                         /* client wants all ctcps */
 			client->want_ctcp = 1;
 	                for (tmp = proxy_clients; tmp != NULL; tmp = tmp->next) {
@@ -182,11 +181,13 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 			}
 			proxy_outdata(client, ":%s NOTICE %s :You're now receiving CTCPs sent to %s\n",
 				      client->proxy_address, client->nick,client->listen->ircnet);
-		} else if (g_strcasecmp(args, "CTCP OFF") == 0) {
+		} else if (g_ascii_strcasecmp(args, "CTCP OFF") == 0) {
                         /* client wants proxy to handle all ctcps */
 			client->want_ctcp = 0;
 			proxy_outdata(client, ":%s NOTICE %s :Proxy is now handling itself CTCPs sent to %s\n",
 				      client->proxy_address, client->nick, client->listen->ircnet);
+		} else {
+			signal_emit("proxy client command", 3, client, args, data);
 		}
 		return;
 	}
@@ -200,6 +201,8 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
         /* check if the command could be redirected */
 	if (strcmp(cmd, "WHO") == 0)
 		grab_who(client, args);
+	else if (strcmp(cmd, "WHOWAS") == 0)
+		proxy_redirect_event(client, "whowas", 1, args, -1);
 	else if (strcmp(cmd, "WHOIS") == 0) {
 		char *p;
 
@@ -294,14 +297,13 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 
 static void sig_listen_client(CLIENT_REC *client)
 {
-	char tmpbuf[1024], *str, *cmd, *args;
-	int ret, recvlen;
+	char *str, *cmd, *args;
+	int ret;
 
 	g_return_if_fail(client != NULL);
 
 	while (g_slist_find(proxy_clients, client) != NULL) {
-		recvlen = net_receive(client->handle->handle, tmpbuf, sizeof(tmpbuf));
-		ret = line_split(tmpbuf, recvlen, &str, &client->buffer);
+		ret = net_sendbuffer_receive_line(client->handle, &str, 1);
 		if (ret == -1) {
 			/* connection lost */
 			remove_client(client);
@@ -368,7 +370,7 @@ static void sig_incoming(IRC_SERVER_REC *server, const char *line)
 	g_return_if_fail(line != NULL);
 
 	/* send server event to all clients */
-	g_string_sprintf(next_line, "%s\n", line);
+	g_string_printf(next_line, "%s\n", line);
 }
 
 static void sig_server_event(IRC_SERVER_REC *server, const char *line,
@@ -391,7 +393,7 @@ static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 	while (*args == ' ') args++;
         g_strdown(event);
 
-	signal = server_redirect_peek_signal(server, event, args, &redirected);
+	signal = server_redirect_peek_signal(server, nick, event, args, &redirected);
 	if ((signal != NULL && strncmp(signal, "proxy ", 6) != 0) ||
 	    (signal == NULL && redirected)) {
 		/* we want to send this to one client (or proxy itself) only */
@@ -401,7 +403,7 @@ static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 	}
 
 	if (signal != NULL) {
-                server_redirect_get_signal(server, event, args);
+                server_redirect_get_signal(server, nick, event, args);
 		if (sscanf(signal+6, "%p", &client) == 1) {
 			/* send it to specific client only */
 			if (g_slist_find(proxy_clients, client) != NULL)
@@ -663,6 +665,14 @@ static void read_settings(void)
 	}
 }
 
+static void sig_dump(CLIENT_REC *client, const char *data)
+{
+	g_return_if_fail(client != NULL);
+	g_return_if_fail(data != NULL);
+
+	proxy_outdata(client, data);
+}
+
 void proxy_listen_init(void)
 {
 	next_line = g_string_new(NULL);
@@ -680,6 +690,8 @@ void proxy_listen_init(void)
 	signal_add("message own_private", (SIGNAL_FUNC) sig_message_own_private);
 	signal_add("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
 	signal_add("setup changed", (SIGNAL_FUNC) read_settings);
+
+	signal_add("proxy client dump", (SIGNAL_FUNC) sig_dump);
 }
 
 void proxy_listen_deinit(void)
@@ -697,4 +709,6 @@ void proxy_listen_deinit(void)
 	signal_remove("message own_private", (SIGNAL_FUNC) sig_message_own_private);
 	signal_remove("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
 	signal_remove("setup changed", (SIGNAL_FUNC) read_settings);
+
+	signal_remove("proxy client dump", (SIGNAL_FUNC) sig_dump);
 }
